@@ -13,7 +13,7 @@ from requests import Response
 from requests.adapters import BaseAdapter
 
 from streamlink.exceptions import NoStreamsError
-from streamlink.plugin import Plugin
+from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import useragents, validate
 from streamlink.stream import HLSStream
 from streamlink.utils.url import update_qsd
@@ -101,20 +101,16 @@ class AbemaTVLicenseAdapter(BaseAdapter):
         return
 
 
+@pluginmatcher(re.compile(r"""
+    https?://abema\.tv/(
+        now-on-air/(?P<onair>[^?]+)
+        |
+        video/episode/(?P<episode>[^?]+)
+        |
+        channels/.+?/slots/(?P<slots>[^?]+)
+    )
+""", re.VERBOSE))
 class AbemaTV(Plugin):
-    '''
-    Abema.tv https://abema.tv/
-    Note: Streams are geo-restricted to Japan
-
-    '''
-    _url_re = re.compile(r"""https://abema\.tv/(
-        now-on-air/(?P<onair>[^\?]+)
-        |
-        video/episode/(?P<episode>[^\?]+)
-        |
-        channels/.+?/slots/(?P<slots>[^\?]+)
-        )""", re.VERBOSE)
-
     _CHANNEL = "https://api.abema.io/v1/channels"
 
     _USER_API = "https://api.abema.io/v1/users"
@@ -144,10 +140,6 @@ class AbemaTV(Plugin):
 
     _SLOT_SCHEMA = validate.Schema({"slot": {"flags": {validate.optional("timeshiftFree"): bool}}})
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls._url_re.match(url) is not None
-
     def __init__(self, url):
         super().__init__(url)
         self.session.http.headers.update({'User-Agent': useragents.CHROME})
@@ -212,7 +204,7 @@ class AbemaTV(Plugin):
         jsonres = self.session.http.json(res, schema=self._USER_SCHEMA)
         self.usertoken = jsonres['token']  # for authorzation
 
-        matchresult = self._url_re.match(self.url)
+        matchresult = self.match
         if matchresult.group("onair"):
             onair = matchresult.group("onair")
             if onair == "news-global":
