@@ -1,14 +1,19 @@
+import logging
 import re
+from urllib.parse import parse_qsl, urlparse
 
-from streamlink.plugin import Plugin
+from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.compat import urlparse, parse_qsl
 from streamlink.stream import HLSStream, HTTPStream, RTMPStream
 
+log = logging.getLogger(__name__)
 
+
+@pluginmatcher(re.compile(
+    r"https?://(?:www\.)?dw\.com/"
+))
 class DeutscheWelle(Plugin):
     default_channel = "1"
-    url_re = re.compile(r"https?://(?:www\.)?dw\.com/")
 
     channel_re = re.compile(r'''<a.*?data-id="(\d+)".*?class="ici"''')
     live_stream_div = re.compile(r'''
@@ -52,10 +57,6 @@ class DeutscheWelle(Plugin):
         })
     )
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls.url_re.match(url) is not None
-
     def _create_stream(self, url, quality=None):
         if url.startswith('rtmp://'):
             return (quality, RTMPStream(self.session, {'rtmp': url}))
@@ -73,7 +74,7 @@ class DeutscheWelle(Plugin):
             m = self.channel_re.search(page.text)
             channel = m and m.group(1)
 
-        self.logger.debug("Using sub-channel ID: {0}", channel)
+        log.debug("Using sub-channel ID: {0}".format(channel))
 
         # extract the streams from the page, mapping between channel-id and stream url
         media_items = self.live_stream_div.finditer(page.text)
