@@ -1,20 +1,18 @@
 import logging
 import re
 
-from streamlink.plugin import Plugin
+from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.stream import HLSStream
 
 log = logging.getLogger(__name__)
 
 
+@pluginmatcher(re.compile(
+    r'https?://(?:www\.)?rotana\.net/live'
+))
 class Rotana(Plugin):
-    _url_re = re.compile(r'https?://(?:www\.)?rotana\.net/live')
     _hls_re = re.compile(r'''["'](?P<url>[^"']+\.m3u8)["']''')
 
-    @classmethod
-    def can_handle_url(cls, url):
-        return cls._url_re.match(url) is not None
-
     def _get_streams(self):
         res = self.session.http.get(self.url)
         m = self._hls_re.search(res.text)
