@@ -1,27 +1,33 @@
 import logging
 import re
 import struct
-from collections import OrderedDict, defaultdict, namedtuple
+from collections import OrderedDict, defaultdict
+from concurrent.futures import Future
 from threading import Event
+from typing import List, NamedTuple, Optional, Union
 from urllib.parse import urlparse
 
+# noinspection PyPackageRequirements
 from Crypto.Cipher import AES
+# noinspection PyPackageRequirements
 from Crypto.Util.Padding import unpad
+from requests import Response
 from requests.exceptions import ChunkedEncodingError
 
 from streamlink.exceptions import StreamError
 from streamlink.stream import hls_playlist
 from streamlink.stream.ffmpegmux import FFMPEGMuxer, MuxedStream
+from streamlink.stream.hls_playlist import Key, M3U8, Map, Segment
 from streamlink.stream.http import HTTPStream
 from streamlink.stream.segmented import (SegmentedStreamReader, SegmentedStreamWorker, SegmentedStreamWriter)
-from streamlink.utils import LazyFormatter
+from streamlink.utils import LRUCache, LazyFormatter
 
 log = logging.getLogger(__name__)
-Sequence = namedtuple("Sequence", "num segment")
 
 
-def num_to_iv(n):
-    return struct.pack(">8xq", n)
+class Sequence(NamedTuple):
+    num: int
+    segment: Segment
 
 
 class HLSStreamWriter(SegmentedStreamWriter):
@@ -30,9 +36,10 @@ class HLSStreamWriter(SegmentedStreamWriter):
         kwargs["retries"] = options.get("hls-segment-attempts")
         kwargs["threads"] = options.get("hls-segment-threads")
         kwargs["timeout"] = options.get("hls-segment-timeout")
-        SegmentedStreamWriter.__init__(self, reader, *args, **kwargs)
+        super().__init__(reader, *args, **kwargs)
 
         self.byterange_offsets = defaultdict(int)
+        self.map_cache: LRUCache[Sequence.segment.map.uri, Future] = LRUCache(kwargs["threads"])
         self.key_data = None
         self.key_uri = None
         self.key_uri_override = options.get("hls-segment-key-uri")
@@ -44,7 +51,11 @@ class HLSStreamWriter(SegmentedStreamWriter):
             segments = "|".join(map(re.escape, ignore_names))
             self.ignore_names = re.compile(rf"(?:{segments})\.ts", re.IGNORECASE)
 
-    def create_decryptor(self, key, sequence):
+    @staticmethod
+    def num_to_iv(n: int) -> bytes:
+        return struct.pack(">8xq", n)
+
+    def create_decryptor(self, key: Key, num: int) -> AES:
         if key.method != "AES-128":
             raise StreamError("Unable to decrypt cipher {0}", key.method)
 
@@ -72,53 +83,83 @@ class HLSStreamWriter(SegmentedStreamWriter):
             self.key_data = res.content
             self.key_uri = key_uri
 
-        iv = key.iv or num_to_iv(sequence)
+        iv = key.iv or self.num_to_iv(num)
 
         # Pad IV if needed
         iv = b"\x00" * (16 - len(iv)) + iv
 
         return AES.new(self.key_data, AES.MODE_CBC, iv)
 
-    def create_request_params(self, sequence):
+    def create_request_params(self, segment: Union[Segment, Map]):
         request_params = dict(self.reader.request_params)
         headers = request_params.pop("headers", {})
 
-        if sequence.segment.byterange:
-            bytes_start = self.byterange_offsets[sequence.segment.uri]
-            if sequence.segment.byterange.offset is not None:
-                bytes_start = sequence.segment.byterange.offset
+        if segment.byterange:
+            bytes_start = self.byterange_offsets[segment.uri]
+            if segment.byterange.offset is not None:
+                bytes_start = segment.byterange.offset
 
-            bytes_len = max(sequence.segment.byterange.range - 1, 0)
+            bytes_len = max(segment.byterange.range - 1, 0)
             bytes_end = bytes_start + bytes_len
-            headers["Range"] = "bytes={0}-{1}".format(bytes_start, bytes_end)
-            self.byterange_offsets[sequence.segment.uri] = bytes_end + 1
+            headers["Range"] = f"bytes={bytes_start}-{bytes_end}"
+            self.byterange_offsets[segment.uri] = bytes_end + 1
 
         request_params["headers"] = headers
 
         return request_params
 
-    def fetch(self, sequence, retries=None):
-        if self.closed or not retries:
+    def put(self, sequence: Sequence):
+        if self.closed:
             return
 
+        if sequence is None:
+            self.queue(None, None)
+        else:
+            # always queue the segment's map first if it exists
+            if sequence.segment.map is not None:
+                future = self.map_cache.get(sequence.segment.map.uri)
+                # use cached map request if not a stream discontinuity
+                # don't fetch multiple times when map request of previous segment is still pending
+                if future is None or sequence.segment.discontinuity:
+                    future = self.executor.submit(self.fetch_map, sequence)
+                    self.map_cache.set(sequence.segment.map.uri, future)
+                self.queue(sequence, future, True)
+
+            # regular segment request
+            future = self.executor.submit(self.fetch, sequence)
+            self.queue(sequence, future, False)
+
+    def fetch(self, sequence: Sequence) -> Optional[Response]:
         try:
-            request_params = self.create_request_params(sequence)
+            return self._fetch(sequence.segment, self.stream_data and not sequence.segment.key)
+        except StreamError as err:  # pragma: no cover
+            log.error(f"Failed to fetch segment {sequence.num}: {err}")
 
-            return self.session.http.get(sequence.segment.uri,
-                                         stream=(self.stream_data
-                                                 and not sequence.segment.key),
-                                         timeout=self.timeout,
-                                         exception=StreamError,
-                                         retries=self.retries,
-                                         **request_params)
-        except StreamError as err:
-            log.error(f"Failed to open segment {sequence.num}: {err}")
+    def fetch_map(self, sequence: Sequence) -> Optional[Response]:
+        try:
+            return self._fetch(sequence.segment.map, self.stream_data and not sequence.segment.key)
+        except StreamError as err:  # pragma: no cover
+            log.error(f"Failed to fetch map for segment {sequence.num}: {err}")
+
+    def _fetch(self, segment: Union[Segment, Map], stream: bool) -> Optional[Response]:
+        if self.closed or not self.retries:  # pragma: no cover
             return
 
-    def should_filter_sequence(self, sequence):
+        request_params = self.create_request_params(segment)
+
+        return self.session.http.get(
+            segment.uri,
+            stream=stream,
+            timeout=self.timeout,
+            exception=StreamError,
+            retries=self.retries,
+            **request_params
+        )
+
+    def should_filter_sequence(self, sequence: Sequence) -> bool:
         return self.ignore_names and self.ignore_names.search(sequence.segment.uri) is not None
 
-    def write(self, sequence, *args, **kwargs):
+    def write(self, sequence: Sequence, *args, **kwargs):
         if not self.should_filter_sequence(sequence):
             try:
                 return self._write(sequence, *args, **kwargs)
@@ -133,11 +174,10 @@ class HLSStreamWriter(SegmentedStreamWriter):
             log.info("Filtering out segments and pausing stream output")
             self.reader.filter_event.clear()
 
-    def _write(self, sequence, res, chunk_size=8192):
+    def _write(self, sequence: Sequence, res: Response, is_map: bool):
         if sequence.segment.key and sequence.segment.key.method != "NONE":
             try:
-                decryptor = self.create_decryptor(sequence.segment.key,
-                                                  sequence.num)
+                decryptor = self.create_decryptor(sequence.segment.key, sequence.num)
             except StreamError as err:
                 log.error(f"Failed to create decryptor: {err}")
                 self.close()
@@ -156,25 +196,28 @@ class HLSStreamWriter(SegmentedStreamWriter):
             self.reader.buffer.write(chunk)
         else:
             try:
-                for chunk in res.iter_content(chunk_size):
+                for chunk in res.iter_content(8192):
                     self.reader.buffer.write(chunk)
             except ChunkedEncodingError:
                 log.error(f"Download of segment {sequence.num} failed")
                 return
 
-        log.debug(f"Download of segment {sequence.num} complete")
+        if is_map:
+            log.debug(f"Segment initialization {sequence.num} complete")
+        else:
+            log.debug(f"Segment {sequence.num} complete")
 
 
 class HLSStreamWorker(SegmentedStreamWorker):
     def __init__(self, *args, **kwargs):
-        SegmentedStreamWorker.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.stream = self.reader.stream
 
         self.playlist_changed = False
-        self.playlist_end = None
-        self.playlist_sequence = -1
-        self.playlist_sequences = []
-        self.playlist_reload_time = 15
+        self.playlist_end: Optional[Sequence.num] = None
+        self.playlist_sequence: int = -1
+        self.playlist_sequences: List[Sequence] = []
+        self.playlist_reload_time: float = 6
         self.playlist_reload_time_override = self.session.options.get("hls-playlist-reload-time")
         self.playlist_reload_retries = self.session.options.get("hls-playlist-reload-attempts")
         self.live_edge = self.session.options.get("hls-live-edge")
@@ -188,25 +231,6 @@ class HLSStreamWorker(SegmentedStreamWorker):
         elif self.playlist_reload_time_override not in ["segment", "live-edge"]:
             self.playlist_reload_time_override = 0
 
-        self.reload_playlist()
-
-        if self.playlist_end is None:
-            if self.duration_offset_start > 0:
-                log.debug(f"Time offsets negative for live streams, skipping back {self.duration_offset_start} seconds")
-            # live playlist, force offset durations back to None
-            self.duration_offset_start = -self.duration_offset_start
-
-        if self.duration_offset_start != 0:
-            self.playlist_sequence = self.duration_to_sequence(self.duration_offset_start, self.playlist_sequences)
-
-        if self.playlist_sequences:
-            log.debug(f"First Sequence: {self.playlist_sequences[0].num}; "
-                      f"Last Sequence: {self.playlist_sequences[-1].num}")
-            log.debug(f"Start offset: {self.duration_offset_start}; "
-                      f"Duration: {self.duration_limit}; "
-                      f"Start Sequence: {self.playlist_sequence}; "
-                      f"End Sequence: {self.playlist_end}")
-
     def _reload_playlist(self, text, url):
         return hls_playlist.load(text, url)
 
@@ -241,12 +265,12 @@ class HLSStreamWorker(SegmentedStreamWorker):
         if sequences:
             self.process_sequences(playlist, sequences)
 
-    def _playlist_reload_time(self, playlist, sequences):
+    def _playlist_reload_time(self, playlist: M3U8, sequences: List[Sequence]) -> float:
         if self.playlist_reload_time_override == "segment" and sequences:
             return sequences[-1].segment.duration
         if self.playlist_reload_time_override == "live-edge" and sequences:
             return sum([s.segment.duration for s in sequences[-max(1, self.live_edge - 1):]])
-        if self.playlist_reload_time_override > 0:
+        if type(self.playlist_reload_time_override) is float and self.playlist_reload_time_override > 0:
             return self.playlist_reload_time_override
         if playlist.target_duration:
             return playlist.target_duration
@@ -255,7 +279,7 @@ class HLSStreamWorker(SegmentedStreamWorker):
 
         return self.playlist_reload_time
 
-    def process_sequences(self, playlist, sequences):
+    def process_sequences(self, playlist: M3U8, sequences: List[Sequence]) -> None:
         first_sequence, last_sequence = sequences[0], sequences[-1]
 
         if first_sequence.segment.key and first_sequence.segment.key.method != "NONE":
@@ -278,10 +302,11 @@ class HLSStreamWorker(SegmentedStreamWorker):
             else:
                 self.playlist_sequence = first_sequence.num
 
-    def valid_sequence(self, sequence):
+    def valid_sequence(self, sequence: Sequence) -> bool:
         return sequence.num >= self.playlist_sequence
 
-    def duration_to_sequence(self, duration, sequences):
+    @staticmethod
+    def duration_to_sequence(duration: int, sequences: List[Sequence]) -> int:
         d = 0
         default = -1
 
@@ -297,6 +322,25 @@ class HLSStreamWorker(SegmentedStreamWorker):
         return default
 
     def iter_segments(self):
+        self.reload_playlist()
+
+        if self.playlist_end is None:
+            if self.duration_offset_start > 0:
+                log.debug(f"Time offsets negative for live streams, skipping back {self.duration_offset_start} seconds")
+            # live playlist, force offset durations back to None
+            self.duration_offset_start = -self.duration_offset_start
+
+        if self.duration_offset_start != 0:
+            self.playlist_sequence = self.duration_to_sequence(self.duration_offset_start, self.playlist_sequences)
+
+        if self.playlist_sequences:
+            log.debug(f"First Sequence: {self.playlist_sequences[0].num}; "
+                      f"Last Sequence: {self.playlist_sequences[-1].num}")
+            log.debug(f"Start offset: {self.duration_offset_start}; "
+                      f"Duration: {self.duration_limit}; "
+                      f"Start Sequence: {self.playlist_sequence}; "
+                      f"End Sequence: {self.playlist_end}")
+
         total_duration = 0
         while not self.closed:
             for sequence in filter(self.valid_sequence, self.playlist_sequences):
@@ -325,20 +369,21 @@ class HLSStreamReader(SegmentedStreamReader):
     __worker__ = HLSStreamWorker
     __writer__ = HLSStreamWriter
 
-    def __init__(self, stream, *args, **kwargs):
-        SegmentedStreamReader.__init__(self, stream, *args, **kwargs)
+    def __init__(self, stream):
         self.request_params = dict(stream.args)
-        self.timeout = stream.session.options.get("hls-timeout")
-
-        self.filter_event = Event()
-        self.filter_event.set()
-
         # These params are reserved for internal use
         self.request_params.pop("exception", None)
         self.request_params.pop("stream", None)
         self.request_params.pop("timeout", None)
         self.request_params.pop("url", None)
 
+        self.filter_event = Event()
+        self.filter_event.set()
+
+        timeout = stream.session.options.get("hls-timeout")
+
+        super().__init__(stream, timeout)
+
     def read(self, size):
         while True:
             try:
@@ -397,7 +442,7 @@ class HLSStream(HTTPStream):
     __reader__ = HLSStreamReader
 
     def __init__(self, session_, url, url_master=None, force_restart=False, start_offset=0, duration=None, **args):
-        HTTPStream.__init__(self, session_, url, **args)
+        super().__init__(session_, url, **args)
         self.url_master = url_master
         self.force_restart = force_restart
         self.start_offset = start_offset
@@ -407,7 +452,7 @@ class HLSStream(HTTPStream):
         return f"<HLSStream({self.url!r}, {self.url_master!r})>"
 
     def __json__(self):
-        json = HTTPStream.__json__(self)
+        json = super().__json__()
 
         if self.url_master:
             json["master"] = self.url_master
